import random
import pygame
import os

class BGM():

    sound_directory = "nagonbot/bin/data/sound/"
    bgm_list = []
    current_song = "nagonbot/bin/data/sound/monstruo.wav"
    bgm_switch = False

    def __init__(self):
        self.bgm_list = os.listdir('nagonbot/bin/data/sound')
    
    def pick_song(self):
        picked_random_number = random.randint(0, len(self.bgm_list)-1)
        song_name = self.bgm_list[picked_random_number]
        print("\n曲:", song_name)
        self.current_song = self.sound_directory + self.bgm_list[picked_random_number]


    def play_song(self, switch: bool):
        if switch == True and self.bgm_switch == True:
            return
        self.bgm_switch = switch
        if self.bgm_switch:
            self.pick_song()
            pygame.init()
            pygame.mixer.init(44100,-16,2,512)
            sound = pygame.mixer.Sound(self.current_song)
            sound.set_volume(0.034)
            sound.play(loops=-1)
        else:
            pygame.init()
            pygame.mixer.init(44100,-16,2,512)
            sound = pygame.mixer.Sound(self.current_song)
            sound.set_volume(0.034)
            sound = pygame.mixer.pause()


if __name__ == "__main__":
    """ bgm_switch = False

    while True:
        BGM().play_song(bgm_switch)
        switch = input("Turn on switch? (y/n) \n")
        if switch == "y":
            bgm_switch = True
        else:
            bgm_switch = False
     """
    print(os.listdir('nagonbot/bin/data/sound'))










from nagonbot.bin.tools.function_tools.browser import Browser
from bs4 import BeautifulSoup
import time
import os
import logging


def get_weather () -> str:

    os.environ['WDM_LOG'] = "false"
    logging.getLogger('WDM').setLevel(logging.NOTSET)
    
    try: 
        chrome = Browser()
        chrome.driver.get("https://www.climatempo.com.br/")
        time.sleep(5)
        soup = BeautifulSoup(chrome.driver.page_source, "html.parser")
        chrome.driver.close()


        #Internet scrapping
        temperature = soup.find("span", id = "current-weather-temperature")
        condition = soup.find("span", id = "current-weather-condition")
        condition_text = translate_weather_condition(condition.text)
        if condition_text == "":
            condition_text = "わからないんです。"
        if temperature.text == "":
            return "気温が取れなかったです。すみません。"
        
        return "今、気温は" + temperature.text + "です。\n天気が" + condition_text
    except:
        return "すみません。いまインターネットはなさそうです。"
    

def translate_weather_condition(current_condition : str) -> str:
    
    condition_dictionary = {
                            'Chuva fraca':'弱い雨が降っています',
                            'Nuvens esparsas': '少し曇っています',
                            'Muitas nuvens' : 'すごく曇っています',
                            'Pancada de chuva':'降雨です',
                            'Trovoada':'雷雨です',
                            'Alguma nebulosidade' : 'なかなか曇っています',
                            'Chuva' : '雨です',
                            'Céu encoberto' : '曇っています'
                            }
    try:
        result = condition_dictionary[current_condition]
    except:
        result = ""
    return result



import pickle
import numpy 
import random
from keras.models import Sequential
from keras.layers import Dense, Activation, Dropout
from keras.optimizers import SGD
from nagonbot.bin.training.json_reader import Json_Reader

class ModelTraining():

    outside_run = False

    def __init__(self) -> None:
        

        json_file = Json_Reader()

        words = []
        classes = []
        documents = []
        ignore_letters = ['。', '、', '！', '？', '「', '」','～']



        for koko in json_file.kokoro['kokoro']:                   
            for pattern in koko['patterns']:
                word_list = [word.surface for word in json_file.tagger(pattern)]
                words.extend(word_list)
                documents.append((word_list, koko['tag']))
                if koko['tag'] not in classes:
                    classes.append(koko['tag'])

        #print("\n\n DOCUMENTS:\n",documents)

        classes = sorted(set(classes))

        words = [word for word in words if word not in ignore_letters]
        words = sorted(set(words))

        #print("\n\n Words:\n",words)
        try:
            pickle.dump(words, open('nagonbot/bin/data/model/words.pkl', 'wb'))
            pickle.dump(classes, open('nagonbot/bin/data/model/classes.pkl', 'wb'))
        except:
            pickle.dump(words, open('words.pkl', 'wb'))
            pickle.dump(classes, open('classes.pkl', 'wb'))
            self.outside_run = True


        #print("\n\n Classes:\n",classes)

        training = []
        output_empty = [0] * len(classes)

        for document in documents:
            bag = []
            word_patterns = document[0]
            for word in words:
                bag.append(1) if word in word_patterns else bag.append(0)
            

            output_row = list(output_empty)
            output_row[classes.index(document[1])] = 1
            training.append([bag, output_row])

        random.shuffle(training)
        training = numpy.array(training)

        train_x = list(training[:, 0])
        train_y = list(training[:, 1])

        model = Sequential()
        model.add(Dense(128, input_shape=(len(train_x[0]),), activation='relu'))
        model.add(Dropout(0.5))
        model.add(Dense(64, activation = 'relu'))
        model.add(Dropout(0.5))
        model.add(Dense(len(train_y[0]), activation='softmax'))


        #sgd = SGD(lr=0.01, decay=1e-6, momentum=0.9, nesterov=True)
        sgd = SGD(lr=0.01, momentum=0.9, nesterov=True)
        model.compile(loss='categorical_crossentropy', optimizer=sgd, metrics=['accuracy'])

        hist = model.fit(numpy.array(train_x), numpy.array(train_y), epochs=200, batch_size = 5, verbose = 1)
        if not self.outside_run:
            model.save('nagonbot/bin/data/model/nagonmodel.h5', hist)
        else:
            model.save('nagonmodel.h5', hist)
        print("出来ました！")


import sqlite3

class CsvCreator():
    
    first_line = True

    def __init__(self) -> None:
        connection = sqlite3.connect('nagonbot/bin/data/database.db')
        connection_cursor = connection.cursor()
        connection_cursor.execute('SELECT * FROM speech_patterns')
        records = connection_cursor.fetchall()
        connection_cursor.close()
        connection.close()


        csv_file = open('nagonbot/bin/data/SpeechPatterns.csv', "w", encoding='utf-8')
        for line in records:
            if self.first_line:
                self.first_line = False
            else:
                csv_file.write("\n")
            line = str(line[0]), line[1], line[2], line[3]
            line = ",".join(line)
            csv_file.write(line)
        csv_file.close
        
        



if __name__ == '__main__':
    CsvCreator()


import pickle
from keras.models import load_model
from keras.layers import Dense, Activation, Dropout
from keras.optimizers import SGD
from importlib import resources

class Model():
    words = None
    classes = None
    model = None

    def __init__(self) -> None:

        #IMPORT LIB
        """ words_pickle = resources.open_binary('nagonbot.bin.data.model', 'words.pkl').read()
        self.words = pickle.load(open(file = words_pickle, mode ='rb'))
        classes_pickle = resources.open_binary('nagonbot.bin.data.model', 'classes.pkl').read()
        self.classes = pickle.load(open(classes_pickle, 'rb'))
        model_file = resources.open_binary('nagonbot.bin.data.model', 'nagonmodel.h5').read()
        self.model = load_model(model_file) """

        #ORIGINAL
        try:
            self.words = pickle.load(open('nagonbot/bin/data/model/words.pkl', 'rb'))
            self.classes = pickle.load(open('nagonbot/bin/data/model/classes.pkl', 'rb'))
            self.model = load_model('nagonbot/bin/data/model/nagonmodel.h5')
        except:
            self.words = pickle.load(open('words.pkl', 'rb'))
            self.classes = pickle.load(open('classes.pkl', 'rb'))
            self.model = load_model('nagonmodel.h5')


from .date_time import *
from .dictionary_bs4 import *
from .weather import *
import random


class Selector():

    def __init__(self):
        pass

    def select(self, tag, i):
        if tag in ["時間" , "日付" , "曜日"]:
                return get_time(tag)
        if tag == "辞書":
            return get_word_dic()
        if tag == "天気":
            Voice().voice_answer("少々お待ちください。\n")
            return  get_weather()
        if tag == "何歳":
            return age()
        return random.choice(i['responses'])
        

    def __repr__(self) -> str:
        return "Class that returns the most approppriate function or response to the identified label."


import numpy 
from nagonbot.bin.tools.function_selector import Selector


class Sentence():

    def clean_up_sentece(self, tagger, sentence):
        sentence_words = [word.surface for word in tagger(sentence)]
        return sentence_words

    def bag_of_words(self, sentence, words, tagger):
        sentence_words = self.clean_up_sentece(tagger, sentence)
        bag = [0] * len(words)
        for w in sentence_words:
            for i, word in enumerate(words):
                if word == w:
                    bag[i] = 1
        return numpy.array(bag)


    def predict_class(self, sentence, model, classes, tagger, words):
        bow = self.bag_of_words(sentence, words, tagger)
        res = model.predict(numpy.array([bow]), verbose = 0)[0]
        ERROR_THRESHOLD = 0.25
        results = [[i, r] for i, r in enumerate(res) if r > 0.1]      

        results.sort(key = lambda x: x[1], reverse = True)
        return_list = []
        for r in results:
            if r[1] < 0.5:
                return_list.append({'koko': '分からない', 'probability': str(r[1])})
            else:
                return_list.append({'koko': classes[r[0]], 'probability': str(r[1])})
        return return_list


    def get_response(self, kokoro_list, kokoro_json):
        tag = kokoro_list[0]['koko']               
        list_of_kokoro = kokoro_json['kokoro']
        for i in list_of_kokoro:
            if i['tag'] == tag:
                return Selector().select(tag, i)


from datetime import datetime

#Funções relacionadas a datas e tempo (retiradas do sistema)
def get_time (tag : str) -> str:
    if tag == "時間":
        str = "今は" + datetime.now().strftime("%H:%M") + "です。"
    if tag == "日付": 
        str = "今日は" + datetime.now().strftime("%Y-%m-%d") + "です。"
    if tag == "曜日":
        if int(datetime.now().strftime("%w")) == 0:
            str = "今日は日曜日です。"
        if int(datetime.now().strftime("%w")) == 1:
            str = "今日は月曜日です。"
        if int(datetime.now().strftime("%w")) == 2:
            str = "今日は火曜日です。"
        if int(datetime.now().strftime("%w")) == 3:
            str = "今日は水曜日です。"
        if int(datetime.now().strftime("%w")) == 4:
            str = "今日は木曜日です。"
        if int(datetime.now().strftime("%w")) == 5:
            str = "今日は金曜日です。"
        if int(datetime.now().strftime("%w")) == 6:
            str = "今日は土曜日です。"
    return str

def age () -> str:
    return str((int(datetime.now().strftime("%Y"))-996)) + "歳です"


from gtts import gTTS
import os
from playsound import playsound
import random

class Voice():

    greetings_list = ['ハローワールド', 'わ、起きちゃった']

    def __repr__(self):
        return "TTS class that is configurated to use google translate's japanese voice as the default voice."

    def start(self):
        picked_random_number = random.randint(0, len(self.greetings_list)-1)
        greeting = self.greetings_list[picked_random_number]
        print("\nナゴン:\n" + greeting)
        self.say(greeting)

    def say(self, message):
        my_aud = gTTS(text = message, lang = 'ja') #converts the text into speech
        try:
            my_aud.save('nagonbot/bin/data/sound/temp/voice.mp3') #save the file with .mp3 extension
            playsound(sound = 'nagonbot/bin/data/sound/temp/voice.mp3', block = True) #to play it
            os.remove('nagonbot/bin/data/sound/temp/voice.mp3')
        except:
            my_aud.save('voice.mp3') #save the file with .mp3 extension
            playsound(sound = 'voice.mp3', block = True) #to play it
            os.remove('voice.mp3')

    def voice_answer(self, answer: str):
        print("\nナゴン:\n" + answer)
        self.say(answer)


from selenium import webdriver
from webdriver_manager.chrome import ChromeDriverManager
import logging
import os


class Browser:

    def __init__(self):
        os.environ['WDM_LOG'] = "false" 
        logging.getLogger('WDM').setLevel(logging.NOTSET)
        options = webdriver.ChromeOptions()
        options.add_experimental_option('excludeSwitches', ['enable-logging'])
        options.add_argument("headless")
        #options.add_experimental_option('detach', True)
        self.driver = webdriver.Chrome(options = options, executable_path = ChromeDriverManager().install())


    def __repr__(self):
        return "Chrome browser that connects to the internet in the background and does not display messages in the console"


import os
from nagonbot.bin.tools.basic_sentence_functions import Sentence
from nagonbot.bin.tools.voice import Voice
#from nagonbot.bin.tools.bgm import BGM
from nagonbot.bin.training.json_reader import Json_Reader
from nagonbot.bin.training.model import Model
from nagonbot.bin.training.training import ModelTraining
from nagonbot.bin.training.csv_to_json import KokoroUpdaterC
import logging


class Main():

    bgm_switch = False
    bgm_is_playing = False
    outside_run = False

    def __init__(self):
        KokoroUpdaterC()
        model_training = ModelTraining()
        if model_training.outside_run:
            self.outside_run = True
        os.environ['WDM_LOG'] = "false"
        logging.getLogger('WDM').setLevel(logging.NOTSET)
        json_file = Json_Reader()
        model = Model()
        os.system('cls')
        Voice().start()
        return self.main(json_file, model)

    def main(self, json_file, model):        
        while True:
            """ if self.bgm_is_playing == False:
                BGM().play_song(self.bgm_switch)
                if self.bgm_switch == True:
                    self.bgm_is_playing = True """
            message = input("\n自分:\n")
            ints = Sentence().predict_class(message, model.model, model.classes, json_file.tagger, model.words)
            response = Sentence().get_response(ints, json_file.kokoro)
            Voice().voice_answer(response)
            if ints[0]['koko']  in ["さようなら" , "寝るさようなら"]:
                break
            """ if ints[0]['koko'] == "BGM_ON":
                if self.bgm_switch == False:
                    self.bgm_switch = True
            if ints[0]['koko'] == "BGM_OFF":
                if self.bgm_switch == True:
                    self.bgm_switch = False
                    if self.bgm_is_playing:
                        self.bgm_is_playing = False """

if __name__ == '__main__':
    Main()


import requests
from bs4 import BeautifulSoup
from .voice import * 

def get_word_dic () -> str:
    Voice().voice_answer("検索したい言葉を教えてください: ")
    search_term = input("\n自分:\n")
    url = "https://www.weblio.jp/content/" + search_term


    page = requests.get(url)
    soup = BeautifulSoup(page.content, "html.parser")

    titles = soup.find_all(class_ = "midashigo")
    descriptions = soup.find_all(class_ = "Sgkdj")

    try:
        number_of_entries = len(descriptions)
        if number_of_entries == 0:
           descriptions = soup.find_all(class_ = "Jtnhj")
           number_of_entries = len(descriptions)
        if number_of_entries == 0:
            return search_term +"という言葉は見つかれなかったです"    
    except:
        return search_term +"という言葉は見つかれなかったです"
    

    try:
        print("\n\n辞典:\n")
        for i in range (0, number_of_entries):
            print(titles[i].text + "\n")
            print(descriptions[i].text + "\n")
    except:
        return search_term +"という言葉は見つかれなかったです"
    return search_term + "という言葉はきっと面白いですね"


import fugashi
import json
from importlib import resources


class Json_Reader():

    tagger = None
    kokoro = None

    def __init__(self) -> None:
        self.tagger = fugashi.Tagger()
        try:
            self.kokoro = json.loads(open("nagonbot/bin/data/kokoro.json", encoding="utf8").read())
        except:
            self.kokoro = json.loads(open("kokoro.json", encoding="utf8").read())
        #self.kokoro = json.load(file)

        #IMPORT LIB
        #self.kokoro = json.load(resources.open_text('nagonbot.bin.data', 'kokoro.json', encoding="utf-8"))

            
        #ORIGINAL
        #self.kokoro = json.loads(open("nagonbot/bin/data/kokoro.json", encoding="utf8").read())

if __name__ == '__main__':
    file = Json_Reader()
    print(file.kokoro)


import json
from importlib import resources

class KokoroUpdaterC():

    def __init__(self) -> None:
        csv_file = resources.open_text('nagonbot.bin.data', "SpeechPatterns.csv", encoding='utf-8').read()
        lines = csv_file.split("\n")
        records = list()
        for line in lines:
            line = line.split(',')
            line[0] = int(line[0])
            records.append(line)

        tag_list = []

        for row in records:
            tag_list.append(row[1])

        unique_tags = set(tag_list)

        list_for_json = []

        for subject in unique_tags:
            pattern_list = []
            response_list = []
            for row in records:
                if row[1] == subject:
                    if row[2] == 'pattern':
                        pattern_list.append(row[3])
                    if row[2] == 'response':
                        response_list.append(row[3])
            json_entry = {"tag": subject, "patterns" : pattern_list, "responses" :response_list}
            list_for_json.append(json_entry)

        json_dict = {"kokoro" : list_for_json}

        try:
            with open("nagonbot/bin/data/kokoro.json", mode = "w", encoding = "utf8") as outfile:
                json.dump(json_dict, outfile, ensure_ascii=False, indent=2)
        except:
            with open("kokoro.json", mode = "w", encoding = "utf8") as outfile:
                json.dump(json_dict, outfile, ensure_ascii=False, indent=2)

        print("心が更新いたしました！！")


if __name__ == '__main__':
    KokoroUpdaterC()


#py setup.py sdist bdist_wheel
#py -m build
#py -m twine upload --verbose --repository testpypi dist/*
#py -m twine upload dist/*

from setuptools import setup, find_packages


setup(
    packages = find_packages(exclude=('tests','docs'))
)


from bs4 import BeautifulSoup
import requests

#Acessando os dados da estação de São Caetano do Sul
#Cabeçalho para burlar a restrição de acesso do SAISP
headers= {'user-agent': 
          'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36',
          'referer': 'https://www.saisp.br/geral/processo.jsp?USERID=Publico&PRODUTO=50',
          'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
          'accept-encoding': 'gzip, deflate, br',
          'accept-language': 'pt-BR,pt;q=0.9,en-US;q=0.8,en;q=0.7'}

#Acessando a página do SAISP para obter os dados
url = 'https://www.saisp.br/geral/processo.jsp?PRODUTO=50&USERID=Publico&&WHICHCHANNEL=143&WHICHCODE=100'

#GET request para obter o conteúdo de HTML primário utilizando o cabeçalho
html_conteudo = requests.get(url, headers=headers).text

#Dividindo o conteúdo do HTML
soup = BeautifulSoup(html_conteudo, "html.parser")

#Acessando somente as linhas dos horários
linhas=soup.find("table").find("tbody", {'id':'tbTelemBody'}).findAll("tr", {'onmouseout':"javascript:this.setAttribute('bgColor','#ffffff');"})

print("Estação Prosperidade em São Caetano do Sul: ")
for linha in linhas:
    #Acessa o horário
    hora=linha.td.text
    #Acessa o valor do pluviômetro da estação no horário
    valorPLU=float(linha.td.next_sibling.text.replace("\n",""))
    #Acessa a tendência do pluviômetro
    tendenciaPLU=linha.td.next_sibling.img['alt']
    #Acessa o valor do fluviômetro da estação no horário
    valorFLU=float(linha.td.next_sibling.next_sibling.text.replace('\n',""))
    #Acessa a tendência do nível fluvial
    tendenciaFLU=linha.td.next_sibling.next_sibling.img['alt']
    print([hora,valorPLU,tendenciaPLU,valorFLU,tendenciaFLU])
    
#Quebra-linha   
print("")  
 
#Agora para a estação de Mauá
url = 'https://www.saisp.br/geral/processo.jsp?PRODUTO=50&USERID=Publico&&WHICHCHANNEL=1000490&WHICHCODE=100'

html_conteudo = requests.get(url, headers=headers).text

soup = BeautifulSoup(html_conteudo, "html.parser")

linhas=soup.find("table").find("tbody", {'id':'tbTelemBody'}).findAll("tr", {'onmouseout':"javascript:this.setAttribute('bgColor','#ffffff');"})

print("Estação da nascente em Mauá: ")
for linha in linhas:
    hora=linha.td.text
    valorPLU=float(linha.td.next_sibling.text.replace("\n",""))
    tendenciaPLU=linha.td.next_sibling.img['alt']
    valorFLU=float(linha.td.next_sibling.next_sibling.text.replace('\n',""))
    tendenciaFLU=linha.td.next_sibling.next_sibling.img['alt']
    
    print([hora,valorPLU,tendenciaPLU,valorFLU,tendenciaFLU])